<?php

namespace Bydedi\Module\Listing;

/**
 * Class ListingAbstract
 * @package Bydedi\Module\Listing\Service
 */
abstract class ListingAbstract implements ListingInterface
{
    /**
     * @var array
     */
    protected $items;
    /**
     * @var int
     */
    protected $page;
    /**
     * @var int
     */
    protected $limit;
    /**
     * @var ItemLoaderInterface
     */
    protected $itemLoader;

    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    protected $request;

    /**
     * @var CriteriaInterface[]
     */
    protected $criteria;

    /**
     * ListingAbstract constructor.
     * @param ItemLoaderInterface $itemLoader
     */
    public function __construct(ItemLoaderInterface $itemLoader)
    {
        $this->reset();
        $this->setItemLoader($itemLoader);
    }

    /**
     * @return mixed|void
     */
    public function reset()
    {
        $this->page = 1;
        $this->limit = null;
        $this->request = null;
        $this->items = null;
        $this->criteria = array();
    }

    /**
     * @param array $param
     * @return bool|void
     */
    public function loadItems($param = array())
    {
        $param = $this->prepareCriteria();
        $this->items = $this->itemLoader->getSearch($param);
    }

    /**
     * Get the items match with the criteria
     *
     * @return array
     */
    public function getItemsToDisplay()
    {
        if (is_null($this->items)) {
            $this->loadItems();
        }

        $offset = $this->limit * ($this->page - 1);
        $itemsSlice = array_slice($this->items, $offset, $this->limit);

        $itemsToDisplay = array();
        foreach ($itemsSlice as $is) {
            $param = array(
                $this->itemLoader->getPrimaryKey() => $is[$this->itemLoader->getPrimaryKey()],
            );

            list($item) = $this->itemLoader->get($param);
            $itemsToDisplay[] = $item;
        }

        return $itemsToDisplay;
    }

    /**
     * @return int
     */
    public function getCountItems()
    {
        return count($this->items);
    }

    /**
     * @return array|mixed
     */
    public function getPagination()
    {
        $pagination = array();

        $total = $this->getCountItems();
        $first = ($this->page - 1) * $this->limit + 1;
        if ($first > $total) {
            $first = $total;
        }
        $last = ($this->page) * $this->limit;
        if ($last > $total) {
            $last = $total;
        }
        $pages = ceil($total / $this->limit);

        $pas = 10;
        $tronc = floor($this->page / $pas);

        if ($tronc == 0) {
            $pagination['INTMIN'] = 1;
            $pagination['INTMAX'] = 9;
            $pagination['PAS'] = $pas - 1;
        } else {
            $pagination['INTMIN'] = $tronc * $pas;
            $pagination['INTMAX'] = ($tronc * $pas) + $pas;
            $pagination['PAS'] = $pas;
        }

        // Affect des donnees de pagination
        $pagination['NBROW'] = $total;
        $pagination['FIRSTROW'] = $first;
        $pagination['LASTROW'] = $last;
        $pagination['NBPAGE'] = $pages;
        $pagination['CURRENTPAGE'] = ($this->page > $pages) ? $pages : $this->page;

        if ($total > 0) {
            $pagination['start'] = ($pagination['CURRENTPAGE'] - 3 < 1) ? 1 : $pagination['CURRENTPAGE'] - 3;
            $pagination['end'] = ($pagination['start'] + 5 > $pagination['NBPAGE']) ?
                $pagination['NBPAGE'] : $pagination['start'] + 5;
        } else {
            $pagination['start'] = 1;
            $pagination['end'] = 1;
        }

        $pagination['prec'] = $pagination['CURRENTPAGE'] - 1;
        $pagination['suiv'] = $pagination['CURRENTPAGE'] + 1;

        return $pagination;
    }

    /**
     * @param int $page
     * @return ListingAbstract
     */
    public function setPage($page)
    {
        $this->page = $page;
        return $this;
    }

    /**
     * @param int $limit
     * @return ListingAbstract
     */
    public function setLimit($limit)
    {
        $this->limit = $limit;
        return $this;
    }

    /**
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @return ListingAbstract
     */
    public function setRequest($request)
    {
        $this->request = $request;
        return $this;
    }

    /**
     * @return int
     */
    public function getPage()
    {
        return $this->page;
    }

    /**
     * @return int
     */
    public function getLimit()
    {
        return $this->limit;
    }

    /**
     * @return array
     */
    public function getItems()
    {
        if (is_null($this->items)) {
            $this->loadItems();
        }
        return $this->items;
    }

    /**
     * @param ItemLoaderInterface $itemLoader
     * @return ListingAbstract
     */
    public function setItemLoader(ItemLoaderInterface $itemLoader)
    {
        $this->itemLoader = $itemLoader;
        return $this;
    }

    /**
     * @param $name
     * @param CriteriaInterface $value
     *
     * @return $this
     */
    public function addCriteria($name, CriteriaInterface $value)
    {
        $this->criteria[$name][] = $value;
        return $this;
    }

    /**
     * @param $name
     * @param CriteriaInterface|CriteriaInterface[] $value
     *
     * @return $this
     */
    public function setCriteria($name, $value)
    {
        $this->criteria[$name] = $value;
        return $this;
    }

    /**
     * @return CriteriaInterface|CriteriaInterface[]
     */
    public function getCriteria()
    {
        return $this->criteria;
    }

    /**
     * @return array
     */
    public function prepareCriteria()
    {
        $param = array();
        foreach ($this->criteria as $index => $criteria) {
            if (is_array($criteria)) {
                $param[$index] = array();
                foreach ($criteria as $c) {
                    $param[$index][] = $c->getValue();
                }
            } else {
                $param[$index] = $criteria->getValue();
            }
        }

        return $param;
    }
}
